/* Custom Styles for Breathe Batteries MkDocs Theme */

/* Background and Text */
.md-content {
    background-color: #ffffff; /* Light background */
    color: #333333;            /* Dark gray text */
}

/* Header */
.md-header {
    background-color: #ffffff;
    color: #333333;
    font-family: 'Roboto', sans-serif;
    font-weight: 300;
}

/* Navigation Links */
.md-nav__link, .md-nav__item {
    color: #333333;
    font-family: 'Roboto Mono', monospace; /* Similar to the tech font on the site */
}

/* Footer */
.md-footer {
    background-color: #ffffff;
    color: #333333;
}

/* Buttons */
.md-button {
    background-color: #333333;
    color: #ffffff;
    font-weight: bold;
    border-radius: 2px;
    padding: 8px 16px;
}

/* Centering of Main Headings */
h1, h2, h3, h4, h5, h6 {
    text-align: center;
    font-family: 'Roboto Mono', monospace; /* Matches website font style */
}

/* Other Customizations */
a {
    color: #333333;
    text-decoration: underline;
}
